/*  class declarations for sender thread parameters, capture thread parameters,
    receiver thread parameters, video capture object.

    Columbia Video Network Tool (CoVit)
    Copyright (C) 2009  Subhrendu Sarkar
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef INCLUDE_COVIT_H
#define INCLUDE_COVIT_H

#include <cstdlib>
#include <stdio.h>
#include <iostream>
#include <ffmpeg/avcodec.h>
#include <ffmpeg/avformat.h>
#include <SDL/SDL.h>
#include <SDL/SDL_thread.h>

//#include "rtp_api.h"
//#include "rtp_highlevel.h"

#include "interfaces.h"
#include "unicapVidCap.h"
//#include "mediabuffer.h"

/*
This is a class that represents a Media buffer and contains pointers to the next and previous Media Buffers. 
*/
class CMediaBuffer
{
private	:
  //the buffer which holds the media data
	char *dataBuffer;
	//the timestamp increment associated with the media buffer in millisecs
	int tsinc_msec;	
  //size of the media buffer
	int buffer_size;
	//indicates if the encoded media buffer is an I frame 
  int iframe;
	

public :
  //count of the buffer out of the 30 media buffers
	int bufferCount;
	//pointer to next Media Buffer
	CMediaBuffer *ObjNextBuffer;
  //pointer to previous Media Buffer
	CMediaBuffer *ObjPrevBuffer;
  //constructor of Media Buffer, it takes as input an integer which is the count of the buffer	
  CMediaBuffer(int x)
	{
		bufferCount = x;
		dataBuffer = NULL;
		ObjNextBuffer = NULL;
		ObjPrevBuffer = NULL;
		iframe = 0;
	};
	//destructor
	~CMediaBuffer()
	{
		bufferCount = 0;
		freeBuffer();
		dataBuffer = NULL;
		ObjNextBuffer = NULL;
		ObjPrevBuffer = NULL;
	};
  //returns the pointer to the media buffer data and sets the size and time stamp incremenet associated with the media buffer
	char *getBuffer(int *size,int *timestampInc)
	{
		*size = buffer_size; 
		
		if(timestampInc != NULL)
			*timestampInc = tsinc_msec; 
		return dataBuffer; 
	};
  //free the media bufffer data
	void freeBuffer()
	{
		free(dataBuffer);
		dataBuffer = NULL;
	};	
	//allocates the media buffer data and copies the data to media buffer data and the time stamp increment; returns 0 on SUCCESS, -1 on FAILURE
  int setBuffer(int size, char *buf, int i_frame, int ts_msec)
	{
		dataBuffer = (char *)malloc(size * sizeof(char));
		if(dataBuffer == NULL)
		{
			return -1;		
		}
		memcpy(dataBuffer,buf, size);
		buffer_size = size;
		tsinc_msec = ts_msec;
		iframe = i_frame;
		return 0;

	};
	//sets the next Media Buffer
  void setNextBuffer(CMediaBuffer *nxtbuf)
	{
		ObjNextBuffer = nxtbuf;	
	};	
	//sets the previous Media Buffer
  void setPrevBuffer(CMediaBuffer *prevbuf)
	{
		ObjPrevBuffer = prevbuf;
		
	};
};

//Sender thread params that are passed to the sender thread when it is created
class CSenderThreadParams
{
public:
  //thread id of the sender thread
	int threadid;
  //mutex conditionals for the 30 Empty Media Buffers shared with Capture thread
	pthread_cond_t *MediaBuffer_Empty_cond[30];
	//mutex conditionals for the 30 Full Media Buffers shared with Capture thread
  pthread_cond_t *MediaBuffer_Full_cond[30];
  //mutex for the 30 Media Buffers shared with Capture thread
	pthread_mutex_t *MediaBuffer_mutex[30];
  //head of Media Buffer object linked list 
	CMediaBuffer *MediaBuffer;
  //host name
	char host[256];
  //port
	int port;
  //time to live for socket connection
	int ttl;	

  //sender thread mutex
	pthread_mutex_t *thr_mutex; 
	//sender thread conditional
  pthread_cond_t  *thr_cond; 

  //sender thread stop mutex
 	pthread_mutex_t *sendstop_cond_mutex;
	//sender thread stop conditional
  pthread_cond_t  *sendstop_cond;

  //status of the sender thread mutex
	pthread_mutex_t *sendstatus_mutex;
  //variable to quit the sender thread externally
	int *send_quit;
};

//Capture thread parameters that are passed to the sender thread when it is created 
class CCapThreadParams
{
	public:
  //mutex conditionals for the 30 Empty Media Buffers shared with sender thread
	pthread_cond_t *MediaBuffer_Empty_cond[30];
	//mutex conditionals for the 30 Full Media Buffers shared with sender thread
  pthread_cond_t *MediaBuffer_Full_cond[30];
  //mutex for the 30 Empty Media Buffers shared with sender thread 
	pthread_mutex_t *MediaBuffer_mutex[30];

  //mutex conditionals for the Empty Media Buffers for deletion 
	pthread_cond_t *m_delMediaBuffer_Empty_cond;
  //mutex conditionals for the Full Media Buffers for deletion 
	pthread_cond_t *m_delMediaBuffer_Full_cond;
  //mutex conditionals for the Media Buffers for deletion 
	pthread_mutex_t *m_delMediaBuffer_mutex;
  //head of Media Buffers linked list shared with the sender thread 
	CMediaBuffer *MediaBuffer;

  //capture thread mutex
	pthread_mutex_t *thr_mutex;
  //capture thread conditional
	pthread_cond_t  *thr_cond;
  //capture thread status mutex
	pthread_mutex_t *capstatus_mutex;

  //mutex for stopping capture thread
	pthread_mutex_t *thrstop_mutex;
  //conditional to signal stopping of capture thread 
	pthread_cond_t *thrstop_cond;
  //variable to stop capture thread externally
	int *stopCapStatus;
  //capture thread identifier
	int threadid;
};

//Receiver thread parameters that are passed to the sender thread when it is created 
class CRecvThreadParams
{
public:
	//CMediaBuffer *MediaBuffer;
  //host name for the receiver
	char host[256];
  //listening port for the receiver
	int port;
  //time to live for the receiver
	int ttl;	
	//receiver thread id	
	int threadid;
  //receiver thread mutex
	pthread_mutex_t *thr_mutex;
  //receiver thread conditional
	pthread_cond_t  *thr_cond; 
  //mutex for stopping receiver thread 
	pthread_mutex_t *recvstop_mutex; 
  //conditional to signal stopping of receiver thread
	pthread_cond_t  *recvstop_cond; 
  //variable to stop receiver thread externally 
	int *recv_quit;
};

//Video Encoder Class 
class CVideoEnc : public IVideoEnc
{
public	:
  //constructor
	CVideoEnc(){};
  //destructor
	virtual ~CVideoEnc(){};
	//encode a frame
  virtual void encode_frame();
	//convert the image format
  virtual void convert_img_format();
};

//Video Capture Class
class CVideoCapture : public IVideoCapture
{
protected :
  //UnicapVidCap object
	CUnicapVidCap *ObjVidCap;
  //display video window rectangle 
	SDL_Rect rect;
  //SDL surface for video display
	SDL_Surface *m_screen;
  //video capture stop status
	int m_stopStatusBase;
  //capture status mutex
	pthread_mutex_t *m_capstatus_mutex;
  //Capture thread stop mutex
	pthread_mutex_t *m_thrstop_mutex;
  //conditional to signal the stopping the capture thread 
	pthread_cond_t *m_thrstop_cond;	

	//mutex conditionals for the 30 Empty Media Buffers shared with sender thread
  pthread_cond_t *m_MediaBuffer_Empty_cond[30];
  //mutex conditionals for the 30 Full Media Buffers shared with sender thread
	pthread_cond_t *m_MediaBuffer_Full_cond[30];
  //mutex for the 30 Media Buffers shared with sender thread
	pthread_mutex_t *m_MediaBuffer_mutex[30];

  //mutex conditionals for the Empty Media Buffers for deletion 
	pthread_cond_t *m_delMediaBuffer_Empty_cond;
  //mutex conditionals for the Full Media Buffers for deletion 
	pthread_cond_t *m_delMediaBuffer_Full_cond;
  //mutex for the Media Buffers for deletion 
	pthread_mutex_t *m_delMediaBuffer_mutex;

	//Video encode bitrate
  int m_encode_bitrate;
  //video capture frame rate
	int m_fps;
  //video encoder GOP size
	int m_gopSize;
  //video encoder codec format
	CodecID m_encode_fmt;

public	:
  //capture thread id associated with the object
	int m_threadid;
  //head of the Media Buffer linked list
	CMediaBuffer *m_MediaBuffer; 
  //Video encoder object
	CVideoEnc *VideoEnc;
	//constructor
	CVideoCapture(int threadid);
	//destructor
	virtual ~CVideoCapture();
	//video capture stop status
	int getStopStatus();
  //set the video capture stop status 
	int setStopStatus();
	//set settings for the device number
	virtual void SetSettings(int device_no);
  //process the video
	void Process_Raw_Video(int buffers_ready,int PixFormat, int bufsize,int width, int height,unsigned char *data, int tsinc_usec);	
	//get the video format associated with the video capture object
	virtual void getVideoFormat();
	//set the video format associated with the video capture object
  virtual void setVideoFormat();
	//get the video device properties associated with the video capture object
  virtual void getVideoDeviceProp();
  //set the MediaBuffer to the video capture object
	virtual void setMediaBuffer(CCapThreadParams*);
	//start the capture
  virtual void start_capture(CCapThreadParams *cparam);
	//stop capture
  virtual int stop_capture();
	
};


#endif
